#include "XMC45_INIT.h"

/********************************************************************************************
 * - This is a loader software which is mapped to memory location 0x080C0000 using the
 * linker script file
 * - The loader software will read the SD card which contain a binary file "firmware.bin" which
 * contains the latest firmware to be flash programmed.
 * -
 *********************************************************************************************/


#define MAGIC_KEY 0xA5C3E10F

enum SW_Config {
	SW_CONFIG_Normal = 0x0, /**< Normal boot mode */
	SW_CONFIG_ABM_PSRAM = 0x4, /**< Use ABM Header in PSRAM to boot */
	SW_CONFIG_ABM0 = 0x8, /**< Use ABM Header in Flash at address 0x0800FFE0 */
	SW_CONFIG_ABM1 = 0xC, /**< Use ABM Header in Flash at address 0x0801FFE0 */
	SW_CONFIG_ABM_Fallback = 0xE /**< Fallback modus */
};

typedef struct ABM_Header {
	uint32_t MagicKey; /**< Magic key. Always 0xA5C3E10F */
	uint32_t StartAddress; /**< Start address of the programm to load */
	uint32_t Length; /**< Length of the programm to load. */
	uint32_t ApplicationCRC32; /**< CRC32 Sum of the complete application code */
	uint32_t HeaderCRC32; /**< CRC32 Sum of the four fields before */
} ABM_Header_t;

ABM_Header_t* PSRAM_Header = (ABM_Header_t*)0x1000FFE0;



uint32_t ABM_CalculateHeaderCRC32(const ABM_Header_t * Header);
void ABM_Make_ABM_header(void);
void Reset_to_NormalMode(void);
void Reset_to_ABM1(void);


 int main(void)
 {
	   FL_STATUS_t fl_status;
	   uint32_t i;

	   DAVE_Init();
	   GPIO_Init();

	   XMC_GPIO_SetOutputLow(LED1_ABM); 	// OFF LED
	   XMC_GPIO_SetOutputLow(LED2_PROG); 	// OFF LED

	   /*************************************************************
	    * Enable this function to have a preview of
	    * the ABM header.
	    *************************************************************/
	   //ABM_Make_ABM_header();



	   /*************************************************************
	    * Flash programming will begin here
	    **************************************************************/
#if TESTING
	   fl_status = FL_SUCCESS;

#else
	   fl_status = Flash_loader();
#endif

	   while(1)
	   {

	   	   if(fl_status == FL_SUCCESS)
		   	   {
		   		 while (1)
		   		 {
		   			XMC_GPIO_ToggleOutput(LED2_PROG);
		   			for(i=0;i<0xfffff;i++);
		   			{
		   				if (!XMC_GPIO_GetInput(BUTTON_2))
		   				{
		   					Reset_to_NormalMode();  //This will execute the application software
		   				}
		   			}
		   		 }
		   	   }
		   	   else
		   	   {
		      	 XMC_GPIO_SetOutputHigh(LED2_PROG); 	// ON LED
		   	   }
	   }
 }




 uint32_t ABM_CalculateHeaderCRC32(const ABM_Header_t * Header)
 {
 	SCU_RESET->PRCLR2 = SCU_RESET_PRCLR2_FCERS_Msk;
 	while((SCU_RESET->PRSTAT2 & SCU_RESET_PRSTAT2_FCERS_Msk));
 	FCE->CLC = 0;
 	while((FCE->CLC & FCE_CLC_DISS_Msk));
 	FCE_KE0->CFG = 0;
 	FCE_KE0->CRC = 0;
 	FCE_KE0->IR = *((uint32_t*)Header);
 	FCE_KE0->IR = *((uint32_t*)(((void*)Header)+4));
 	FCE_KE0->IR = *((uint32_t*)(((void*)Header)+8));
 	FCE_KE0->IR = *((uint32_t*)(((void*)Header)+12));
 	return FCE_KE0->RES;
 }

 void ABM_Make_ABM_header(void)
 {
 	PSRAM_Header->MagicKey = MAGIC_KEY;
 	PSRAM_Header->StartAddress = 0x080C0000;		// Update_ABM: Ensure that this loader start address is correct
 	PSRAM_Header->Length = 0xFFFFFFFF;
 	PSRAM_Header->ApplicationCRC32 = 0xFFFFFFFF;
 	PSRAM_Header->HeaderCRC32 = ABM_CalculateHeaderCRC32(PSRAM_Header);
 }

 void Reset_to_NormalMode(void)
 {
 	/* Clear the reset cause field for proper reset detection of the ssw */
 	SCU_RESET->RSTCLR = 1<<SCU_RESET_RSTCLR_RSCLR_Pos;
 	/* Set ABM0 as boot mode in SWCON field of STCON register */
 	WR_REG(SCU_GENERAL->STCON,
 			SCU_GENERAL_STCON_SWCON_Msk,
 			SCU_GENERAL_STCON_SWCON_Pos,
 			SW_CONFIG_Normal);
 	/* request system reset */
 	PPB->AIRCR = 1 << PPB_AIRCR_SYSRESETREQ_Pos |
 			0x5FA<<PPB_AIRCR_VECTKEY_Pos | 0x1 << PPB_AIRCR_PRIGROUP_Pos;
 }

 void Reset_to_ABM1(void)
 {
 	/* Clear the reset cause field for proper reset detection of the ssw */
 	SCU_RESET->RSTCLR = 1<<SCU_RESET_RSTCLR_RSCLR_Pos;
 	/* Set ABM0 as boot mode in SWCON field of STCON register */
 	WR_REG(SCU_GENERAL->STCON,
 			SCU_GENERAL_STCON_SWCON_Msk,
 			SCU_GENERAL_STCON_SWCON_Pos,
 			SW_CONFIG_ABM1);
 	/* request system reset */
 	PPB->AIRCR = 1 << PPB_AIRCR_SYSRESETREQ_Pos |
 			0x5FA<<PPB_AIRCR_VECTKEY_Pos | 0x1 << PPB_AIRCR_PRIGROUP_Pos;
 }








